//
//  led7seg.c
//
//  7-Sengent LED Control
//  Copyright (C) 2009-2011 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		7-Segment LEDs display control subroutine.
//		Supported 2~8digits anode/cathode common type.
//		Any signal porality.
//		Digit signal is continous port bits. ex.dig1=b0 dig2=b1 dig3=b2...
//		Segment signal connection is fixed. a=b0= b=b1 c=b3 ... g=b6 dot=b7
//
//  Device: PIC16F88         IDE: MPLAB IDE 8.36, HI-TECH C Tools for the PIC10/12/16 MCU Family Version 9.60PL5
//  Device: PIC18F14K50      IDE: MPLAB IDE 8.36, C Compiler for PIC18 MCUs v3.33
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2009.04.23 v0.0  1st rel(BDCM version,PIC16F88 only)
//    2011.01.01 v0.1  Support PIC18/C18
//

#include "led7seg.h"

unsigned char led7seg_Dat[LED7SEG_DIG_WIDTH];
unsigned char led7seg_DimSet = 0;
const  unsigned char led7seg_Pat[16] = {
//		  .gfedcba
		0b00111111,		// 0
		0b00000110,		// 1	 aaa
		0b01011011,		// 2	f   b
		0b01001111,		// 3	 ggg
		0b01100110,		// 4	e   c
		0b01101101,		// 5	 ddd .
		0b01111101,		// 6
		0b00000111,		// 7
		0b01111111,		// 8
		0b01101111,		// 9
		0b01110111,		// A
		0b01111100,		// B
		0b00111001,		// C
		0b00111110,		// D
		0b01111001,		// E
		0b01110001		// F
};


void led7seg_PutPattern(unsigned char pat, unsigned char pos){
	if (pos > 0) {
		--pos;
		led7seg_Dat[pos] = pat | (led7seg_Dat[pos] & 0x80);
	} else {
		unsigned char i;

		for(i = 0; i < LED7SEG_DIG_WIDTH; i++) {
			led7seg_Dat[i] = pat | (led7seg_Dat[i] & 0x80);
		}
	}
	return;
}

void led7seg_DP(unsigned char f, unsigned char pos){
	if (pos > 0) {
		--pos;
		if (f) {
			led7seg_Dat[pos] |= 0x80;
		} else {
			led7seg_Dat[pos] &= ~0x80;
		}
	} else {
		unsigned char i;

		for(i = 0; i < LED7SEG_DIG_WIDTH; i++) {
			if (f) {
				led7seg_Dat[i] |= 0x80;
			} else {
				led7seg_Dat[i] &= ~0x80;
			}
		}
	}
	return;
}

void led7seg_Dim(signed char dim){
	if (dim > 0) {
		led7seg_DimSet = 0;				// Max
	} else if (dim == 0) {
		led7seg_DimSet = 16;			// Dark
	} else {
		led7seg_DimSet = 1 << (~dim & 0x07);
	}
	return;
}

void led7seg_DispInt(int iValue, unsigned char bPos, unsigned char bLen) {
	int a;
	unsigned char s;
	signed char l;

	if (iValue < 0) {
		s = 1;
		a = -iValue;
	} else {
		s = 0;
		a = iValue;
	}

	l = bLen - 1;
	do {
		unsigned char b;

		b = a % 10;
		a = a / 10;
		led7seg_PutHex(b, bPos + l);
		--l;
	} while((l >= 0) && (a != 0));

	if (l >= 0) {
		if (s) {
			led7seg_PutPattern(0x40, bPos + l);		// minus pattern
			--l;
		}
		for( ; l >= 0; --l) {
			led7seg_PutPattern(0x00, bPos + l);		// zero suppless
		}
	}

	return;
}


//
// LED port access
//
#define LED7SEG_DIG_BITS	(((((unsigned short)0x00ff << LED7SEG_DIG_WIDTH) & (unsigned short)0xff00) << LED7SEG_DIG_SBIT) >> 8)	// cast(unsigned short) is MPLAB/C18 integer promotion control

#ifdef LED7SEG_DIG_REV
	#if LED7SEG_DIG_POL == 1
	#define	DIG_ON(d)	(LED7SEG_DIG_PORT |=  (1 << (LED7SEG_DIG_SBIT+LED7SEG_DIG_WIDTH-1-(d))))
	#define	DIG_OFF(d)	(LED7SEG_DIG_PORT &= ~(1 << (LED7SEG_DIG_SBIT+LED7SEG_DIG_WIDTH-1-(d))))
	#else
	#define	DIG_ON(d)	(LED7SEG_DIG_PORT &= ~(1 << (LED7SEG_DIG_SBIT+LED7SEG_DIG_WIDTH-1-(d))))
	#define	DIG_OFF(d)	(LED7SEG_DIG_PORT |=  (1 << (LED7SEG_DIG_SBIT+LED7SEG_DIG_WIDTH-1-(d))))
	#endif
#else
	#if LED7SEG_DIG_POL == 1
	#define	DIG_ON(d)	(LED7SEG_DIG_PORT |=  (1 << ((d)+LED7SEG_DIG_SBIT)))
	#define	DIG_OFF(d)	(LED7SEG_DIG_PORT &= ~(1 << ((d)+LED7SEG_DIG_SBIT)))
	#else
	#define	DIG_ON(d)	(LED7SEG_DIG_PORT &= ~(1 << ((d)+LED7SEG_DIG_SBIT)))
	#define	DIG_OFF(d)	(LED7SEG_DIG_PORT |=  (1 << ((d)+LED7SEG_DIG_SBIT)))
	#endif
#endif

#if LED7SEG_SEG_POL == 1
#define SEG_SET(p)	(LED7SEG_SEG_PORT = (p))
#else
#define SEG_SET(p)	(LED7SEG_SEG_PORT = ~(p))
#endif

void led7seg_Start(void){
	unsigned char	i;

	SEG_SET(0x00);							// Port inactive
	LED7SEG_DIG_TRIS &= ~LED7SEG_DIG_BITS;	// Set output
	LED7SEG_SEG_TRIS = 0x00;				// Set output
	for(i = 0; i< LED7SEG_DIG_WIDTH; i++) {
		led7seg_Dat[i] = 0b00110110;
	}
	return;
}

void led7seg_Stop(void){
	SEG_SET(0x00);							// Port inactive
	LED7SEG_DIG_TRIS |= LED7SEG_DIG_BITS;	// Set input
	LED7SEG_SEG_TRIS = 0xff;				// Set input
	return;
}

void led7seg_Refresh(void){
	static unsigned char	loc = 0;
	static signed char		dim = 0;

	DIG_OFF(loc);
	if (--dim < 0) {
		loc++;
		if (loc >= LED7SEG_DIG_WIDTH) {
			loc = 0;
			dim = led7seg_DimSet;
		}
		SEG_SET(led7seg_Dat[loc]);
		DIG_ON(loc);
	}
	return;
}
