//
//  main.c
//
//  DeskClock 0.1
//  Copyright(C)2010-2011 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Digital desk clock.
//		Supply and control by USB.
//
//	Usage
//		Connect USB power adapter or PC.
//		Input now time by push button.
//		The command control from the PC COM port(virtual serial port)
//
//  Device: PIC18F14K50
//
//  Pin assign:
//		p1:  Vdd      <-- USB+5V
//		p2:  RA5      <-- Crystal 12MHz
//		p3:  RA4 AN3  --> Crystal 12MHz
//		p4:  RA3 MCLR <-- SW1
//		p5:  RC5 CCP  --> LED d
//		p6:  RC4 CCP  --> LED e
//		p7:  RC3 AN7  --> LED f
//		p8:  RC6 AN8  --> LED g
//		p9:  RC7      --> LED dp
//		p10: RB7      --> LED dig1 **
//		p11: RB6 SCL  --> LED dig2 **
//		p12: RB5 AN11 --> LED dig3 **
//		p13: RB4 SDA  --> LED dig4 **
//		p14: RC2 AN6  --> LED c
//		p15: RC1 AN5  --> LED b
//		p16: RC0 AN4  --> LED a
//		p17: VUSB     --> C 0.47uF
//		p18: RA1 D-   <-> USB D-
//		p19: RA0 D+   <-> USB D+
//		p20: Vss      <-- GND
//
//	Clock:	Fosc=48MHz
//
//	Internal peripheral module:
//		WDT:	4sec
//		BOR:	-
//		EEPROM:	-
//		Timer0:	LED refresh interval
//		Timer1:	
//		Timer2:	-
//		Timer3:	-
//		Comp:	-
//		ECCP1:	-
//		USB:	Serial emuration(high speed)
//		MSSP:	-
//		EUSART:	-
//		ADC:	-
//		FVR:	-
//
//	Environment:
//		IDE: MPLAB IDE 8.36
//      C Compiler for PIC18 MCUs v3.33
//      Microchip Application Libraries v2009-11-18
//      Standard USB bootloader
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2010.12.31 v0.0  First cording
//    2011.01.16 v0.1  12h/24h, fine adjust, auto dimmer and save setting
//    2011.01.17 v0.1  Add on serial number "#DC01999"
//    2011.01.18 v0.1  Bug fix, do not work echo command
//    2011.01.20 v0.1  Response code "ok/ng 999999"
//

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "p18f14K50.h"					// Microchip C18 library
#include "timers.h"						// Microchip C18 library
#include "EEP.h"						// Microchip C18 library

#include "HardwareProfile.h"			// Microchip Application Libraries v2009-11-18
#include "GenericTypeDefs.h"			// Microchip Application Libraries v2009-11-18
#include "Compiler.h"					// Microchip Application Libraries v2009-11-18
#include "usb_config.h"					// Microchip Application Libraries v2009-11-18
#include "USB/usb.h"					// Microchip Application Libraries v2009-11-18
#include "USB/usb_device.h"				// Microchip Application Libraries v2009-11-18
#include "USB/usb_function_cdc.h"		// Microchip Application Libraries v2009-11-18

#include "led7seg.h"					// Private library
#include "utoanm.h"						// Private library


//
// Application constant
//

// Hardware
#define PUSH_SW			(!PORTAbits.RA3)// 
#define PUSH_SW1_MUX	(LATBbits.LATB4)// -RA3 Multuplexd
#define PUSH_SW2_MUX	(LATBbits.LATB5)// -RA3 Multuplexd

// Initial value
#define TMR0VAL		-46875				// Interval = 1Hz < CLK(48MHz) / 4 / PS(256) >
#define TMR1VAL		-7500				// Interval = 1600Hz < CLK(48MHz) / 4 / PS(1) >
#define HH12		0					// 1:12H 0:24H display
#define DIM_HI_HH	9					// Dimmer bright start hour
#define DIM_LO_HH	17					// Dimmer dark start hour

// Inline function
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define ABS(a)		(((a) >  0 ) ? (a) : -(a))

//
// Configuration bits
//
#pragma config CPUDIV = NOCLKDIV		// 48MHz
#pragma config USBDIV = OFF				// No div
#pragma config FOSC   = HS				// Xtal=12MHz
#pragma config PLLEN  = ON				// Xtal * 4
#pragma config FCMEN  = OFF
#pragma config IESO   = OFF
#pragma config PWRTEN = OFF
#pragma config BOREN  = OFF
#pragma config BORV   = 19
//#pragma config VREGEN = ON
#pragma config WDTEN  = OFF
#pragma config WDTPS  = 1024			// 4sec
#pragma config MCLRE  = OFF
#pragma config HFOFST = OFF
#pragma config STVREN = ON
#pragma config LVP    = OFF
#pragma config XINST  = OFF
#pragma config BBSIZ  = OFF
#pragma config CP0    = OFF
#pragma config CP1    = OFF
#pragma config CPB    = OFF
#pragma config WRT0   = OFF
#pragma config WRT1   = OFF
#pragma config WRTB   = OFF
#pragma config WRTC   = OFF
#pragma config EBTR0  = OFF
#pragma config EBTR1  = OFF
#pragma config EBTRB  = OFF       


//********************************************************************
//
// Global area
//
//********************************************************************

//
// Tables
//
#define MODE_INIT			0
#define MODE_NORMAL			1
#define MODE_SEC			2
#define MODE_ADJ_HH			3
#define MODE_ADJ_MI			4
#define MODE_HOST_N			5
#define MODE_SIZE			6
#define EVENT_MODE_PUSH		0
#define EVENT_MODE_LONG		1
#define EVENT_UP_PUSH		2
#define EVENT_UP_LONG		3
#define EVENT_DOWN_PUSH		4
#define EVENT_DOWN_LONG		5
#define EVENT_SIZE			6
#define EVENT_NONE			99
#define ACTION_NOP			0
#define ACTION_HH_UP		1
#define ACTION_HH_DOWN		2
#define ACTION_MI_UP		3
#define ACTION_MI_DOWN		4
#define ACTION_SS_ZERO		5
#define ACTION_DIM_HI		6
#define ACTION_DIM_LO		7
const rom struct {
	unsigned char	next:4;
	unsigned char	action:4;
} decision_table[MODE_SIZE][EVENT_SIZE] = {
	//                   EVENT_MODE_PUSH               EVENT_MODE_LONG           EVENT_UP_PUSH                EVENT_UP_LONG                EVENT_DOWN_PUSH               EVENT_DOWN_LONG 
	/* MODE_INIT   */ { {MODE_ADJ_HH,ACTION_NOP},     {MODE_ADJ_HH,ACTION_NOP}, {MODE_INIT  ,ACTION_DIM_HI}, {MODE_INIT  ,ACTION_DIM_HI}, {MODE_INIT  ,ACTION_DIM_LO},  {MODE_INIT  ,ACTION_DIM_LO} },
	/* MODE_NORMAL */ { {MODE_SEC   ,ACTION_NOP},     {MODE_ADJ_HH,ACTION_NOP}, {MODE_NORMAL,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_DIM_LO},  {MODE_NORMAL,ACTION_DIM_LO} },
	/* MODE_SEC    */ { {MODE_NORMAL,ACTION_NOP},     {MODE_ADJ_HH,ACTION_NOP}, {MODE_SEC   ,ACTION_DIM_HI}, {MODE_SEC   ,ACTION_DIM_HI}, {MODE_SEC   ,ACTION_DIM_LO},  {MODE_SEC   ,ACTION_DIM_LO} },
	/* MODE_ADJ_HH */ { {MODE_ADJ_MI,ACTION_NOP},     {MODE_ADJ_HH,ACTION_NOP}, {MODE_ADJ_HH,ACTION_HH_UP},  {MODE_ADJ_HH,ACTION_HH_UP},  {MODE_ADJ_HH,ACTION_HH_DOWN}, {MODE_ADJ_HH,ACTION_HH_DOWN}},
	/* MODE_ADJ_MI */ { {MODE_NORMAL,ACTION_SS_ZERO}, {MODE_ADJ_MI,ACTION_NOP}, {MODE_ADJ_MI,ACTION_MI_UP},  {MODE_ADJ_MI,ACTION_MI_UP},  {MODE_ADJ_MI,ACTION_MI_DOWN}, {MODE_ADJ_MI,ACTION_MI_DOWN}},
	/* MODE_HOST_N */ { {MODE_NORMAL,ACTION_NOP},     {MODE_ADJ_HH,ACTION_NOP}, {MODE_NORMAL,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_DIM_LO},  {MODE_NORMAL,ACTION_DIM_LO} }
};

//
// Inter function variables
//
volatile unsigned char	hh = 0;			// ISR: date time counter
volatile unsigned char	mi = 0;			//
volatile unsigned char	ss = 0;			//

volatile unsigned char	sw_bits;		// ISR: Status of push switch
#define SW_BITS_MODE	0x01
#define SW_BITS_UP		0x02
#define SW_BITS_DOWN	0x04
#define SW_MODE			(sw_bits & SW_BITS_MODE)
#define SW_UP			(sw_bits & SW_BITS_UP)
#define SW_DOWN			(sw_bits & SW_BITS_DOWN)

struct {								// EEPROM copy
	unsigned char	magic_code;			// Factory setting ID
	unsigned char	hh12;				// 12h or 24h
	unsigned short	tmr0val;			// 1sec count
	unsigned short	tmr1val;			// LED refresh count
	unsigned char	dim_hi_hh;			//
	unsigned char	dim_lo_hh;			//
	unsigned char	update;				// Use internal
} ee;

//********************************************************************
//
// Interrupt Service Routine
//
//********************************************************************
#pragma code
#pragma interrupt HighPriorityISRCode
void HighPriorityISRCode() {
	if (PIE2bits.USBIE && PIR2bits.USBIF) {
		USBDeviceTasks();
		PIR2bits.USBIF = 0;
	}
}
#pragma interruptlow LowPriorityISRCode
void LowPriorityISRCode() {
	if (PIE1bits.TMR1IE && PIR1bits.TMR1IF) {

//		WriteTimer1(TMR1VAL);			// House keeping
		WriteTimer1(ee.tmr1val);		// House keeping
		PIR1bits.TMR1IF = 0;			//

		// LED scan
		led7seg_Refresh();

		// Key scan
		{
			static unsigned char	stick = 0;

			if (PUSH_SW1_MUX) {
				if (stick && !PUSH_SW) {
					sw_bits = SW_BITS_UP;
				}
			} else if (PUSH_SW2_MUX) {
				if (stick && !PUSH_SW) {
					sw_bits = SW_BITS_DOWN;
				}
			} else {
				if (PUSH_SW) {
					if (stick && (sw_bits == 0)) {
						sw_bits = SW_BITS_MODE;
					}
					stick = 1;
				} else {
					stick = 0;
					sw_bits = 0;
				}
			}
		}

	} else if (INTCONbits.TMR0IE && INTCONbits.TMR0IF) {
//		WriteTimer0(TMR0VAL);			// House keeping
		WriteTimer0(ee.tmr0val);		// House keeping
		INTCONbits.TMR0IF = 0;			//

		ss++;
		if (ss >= 60) {
			ss = 0;
			mi++;
			if (mi >= 60) {
				mi = 0;
				hh++;
				if (hh >= 24) {
					hh = 0;
				}
			}
		}
	}

}

//********************************************************************
//
// Vector address
//
//********************************************************************
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
void _reset (void){
	    _asm goto _startup _endasm
}
#endif
#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void) {
     _asm goto HighPriorityISRCode _endasm
}
#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void) {
     _asm goto LowPriorityISRCode _endasm
}
#pragma code

//********************************************************************
//
// USB Device support (Microchip library)
//
//********************************************************************
// Callbacks
void USBCBSuspend(void) {
}

void USBCBWakeFromSuspend(void) {
}

void USBCB_SOF_Handler(void) {
}

void USBCBErrorHandler(void) {
}

void USBCBStdSetDscHandler(void) {
}

void USBCBCheckOtherReq(void) {
    USBCheckCDCRequest();
}

void USBCBInitEP(void) {
    CDCInitEP();
}

#ifdef DELETE_CODE
void USBCBSendResume(void) {
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do {
        delay_count--;
    } while(delay_count);
    USBResumeControl = 0;
}
#endif 

#if defined(ENABLE_EP0_DATA_RECEIVED_CALLBACK)
void USBCBEP0DataReceived(void) {
}
#endif

BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size) {
    switch(event) {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}

//********************************************************************
//
// Subroutines
//
//********************************************************************
void usb_puts(char *buf) {
	if ((USBDeviceState >= CONFIGURED_STATE) && (USBSuspendControl != 1)) {
		int i;

		for(i = 300; i > 0; --i) {
			if(mUSBUSARTIsTxTrfReady()) break;
			CDCTxService();
		}
		if (i > 0) {
 			putsUSBUSART(buf);
			CDCTxService();
		}
	}
	return;
}

char *usb_gets(char *buf, int sz) {
	static BYTE		p = 0;

	if ((USBDeviceState >= CONFIGURED_STATE) && (USBSuspendControl != 1)) {
		--sz;
		p += getsUSBUSART(&buf[p], sz-p);	// Aveilable an echo command
		if ((p >= sz)          ||
		    (buf[p-1] == '\r') ||
	    	(buf[p-1] == '\n')) {
			if (p < sz) {
				buf[p] = '\0';
			}
			p = 0;
			return(buf);
		}
	}
	return(NULL);
}

//********************************************************************
//
//		M A I N
//
//********************************************************************
void main(void){
	unsigned char	mode;
	short			key_cnt_mode;
	short			key_cnt_up;
	short			key_cnt_down;
	short			host_n;
	unsigned char	host_dp;
	short			i;
	short			hh_s;
	
	//
	// init
	//
	WPUA               = 0;				// Internal pullup default all off
	WPUB               = 0;				//
	WPUAbits.WPUA3     = 1;				// SW1 pullup
	INTCON2bits.RABPU  = 0;				// Enable pullup
//	WDTCONbits.SWDTEN  = 1;				// WDT start for about 1sec sleep
	OSCCONbits.IDLEN   = 1;				// No stop peripherals clock on SLEEP
	INTCONbits.GIE     = 1;
	INTCONbits.PEIE    = 1;
	RCONbits.IPEN      = 1;				// Priority mechanism
	IPR2bits.USBIP     = 1;				// High priority
	INTCON2bits.TMR0IP = 0;				// Low priority
	IPR1bits.TMR1IP    = 0;				// Low priority

	// Recall from EEPROM
	{
		unsigned char	*p;
		unsigned int	a;

		p = (unsigned char *)&ee;
		for(a = 0; a < sizeof(ee); a++){
			*p++ = Read_b_eep(a);
		}
		if (ee.magic_code != 0x55) {
			ee.magic_code = 0x55;
			ee.tmr0val    = TMR0VAL;
			ee.tmr1val    = TMR1VAL;
			ee.hh12       = HH12;
			ee.dim_hi_hh  = DIM_HI_HH;
			ee.dim_lo_hh  = DIM_LO_HH;
			ee.update     = 1;
		}
	}

	// USB
	ADCON1 |= 0x0F;						// Default all pins to digital
	USBDeviceInit();
	USBDeviceAttach();

	// 1sec interval
	OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_SOURCE_INT & T0_PS_1_256);

	// 7-Seg LED & Auto refresh interval
	led7seg_Start();
	OpenTimer1(TIMER_INT_ON & T1_16BIT_RW & T1_SOURCE_INT & T1_PS_1_1 & T1_OSC1EN_OFF & T1_SYNC_EXT_OFF);

	mode         = MODE_INIT;
	key_cnt_mode = 0;
	key_cnt_up   = 0;
	key_cnt_down = 0;
	hh_s         = 24;


	//
	// Main loop
	//
	for(i = 0; ; i++) {
		unsigned char	e;
		char		cmd[30];

		//
		// Push key
		//
		if (SW_MODE) {
			key_cnt_mode++;
		} else {
			key_cnt_mode = 0;
		}
		if (SW_UP) {
			key_cnt_up++;
		} else {
			key_cnt_up = 0;
		}
		if (SW_DOWN) {
			key_cnt_down++;
		} else {
			key_cnt_down = 0;
		}

		if (key_cnt_mode == 100) {
			e = EVENT_MODE_PUSH;
		} else if (key_cnt_mode > 3000) {
			e = EVENT_MODE_LONG;
		} else if (key_cnt_up == 100) {
			e = EVENT_UP_PUSH;
		} else if (key_cnt_up > 3000) {
			e = EVENT_UP_LONG;
		} else if (key_cnt_down == 100) {
			e = EVENT_DOWN_PUSH;
		} else if (key_cnt_down > 3000) {
			e = EVENT_DOWN_LONG;
		} else {
			e = EVENT_NONE;
		}

		//
		// Action
		//
		if (e < EVENT_SIZE) {

			switch(decision_table[mode][e].action) {
			case ACTION_HH_UP:
				hh++;
				if (hh >= 24) {
					hh = 0;
				}
				break;
			case ACTION_HH_DOWN:
				if (hh == 0) {
					hh = 24;
				}
				--hh;
				break;
			case ACTION_MI_UP:
				mi++;
				if (mi >= 60) {
					mi = 0;
				}
				break;
			case ACTION_MI_DOWN:
				if (mi == 0) {
					mi = 60;
				}
				--mi;
				break;
			case ACTION_SS_ZERO:
				ss = 0;
				break;
			case ACTION_DIM_HI:
				led7seg_Dim(1);
				break;
			case ACTION_DIM_LO:
				led7seg_Dim(0);
				break;
			default:
				break;
			}
	
			mode = decision_table[mode][e].next;
		}
		if (hh_s != hh) {
			hh_s = hh;
			if (hh == ee.dim_hi_hh) {
				led7seg_Dim(1);
			}
			if (hh == ee.dim_lo_hh) {
				led7seg_Dim(0);
			}
		}

		//
		// Display
		//
		switch(mode) {
		case MODE_SEC:
			led7seg_DispInt(mi+100, 1, 2);
			led7seg_DispInt(ss+100, 3, 2);
			led7seg_DP(1, 2);
			break;
		case MODE_HOST_N:
			led7seg_DispInt(host_n, 1, 4);
			led7seg_DP(0, 0);
			if (host_dp != 4) {
				led7seg_DP(1, host_dp);
			}
			break;
		default:
//		case MODE_INIT:
//		case MODE_NORMAL:
//		case MODE_ADJ_HH:
//		case MODE_ADJ_MI:
			if (ss & 0x01) {			// Blink colon
				led7seg_DP(1, 2);
			} else {
				led7seg_DP(0, 2);
			}
			if ((mode == MODE_INIT) && (ss & 0x01)) {
				led7seg_PutPattern(0x00, 0);
			} else if ((mode == MODE_ADJ_HH) && (i & 0x80)) {
				led7seg_PutPattern(0x00, 1);
				led7seg_PutPattern(0x00, 2);
			} else if ((mode == MODE_ADJ_MI) && (i & 0x80)) {
				led7seg_PutPattern(0x00, 3);
				led7seg_PutPattern(0x00, 4);
			} else {
				unsigned char	h;

				h = ee.hh12 ? (hh%12) : hh;
				led7seg_DispInt(h, 1, 2);
				led7seg_DispInt(mi+100, 3, 2);
			}
			break;
		}

		//
		// USB Command
		//
   	    if (usb_gets(cmd, sizeof(cmd)-3) != NULL) {
			char			*p;			// Work
			short			v;			// Parameter value
			unsigned char	d;			// Dot point column
			unsigned char	f;			// EEPROM update flag
			unsigned short	r;			// Response code

			p = strchr(cmd, '\0');
			*(p-1) = '\r';
			*p++   = '\n';
			*p     = '\0';
//			strtok(cmd, "\r\n");
//			strcatpgm2ram(cmd, "\r\n");
			usb_puts(cmd);				// Echo back

			d = 0;						// Argument cut in dot
			p = strchr(cmd, '.');
			if (p != NULL) {
				for(; *p != '\0'; p++) {
					*p = *(p+1);
					if (isdigit(*p)) {
						d++;
					}
				}
			}
			v = atoi(&cmd[2]);

			r = 0;
			f = 1;
			if ((cmd[0] == 's') && (cmd[1] == 't')) {
				hh = v / 100;
				mi = v % 100;
				ss = 0;
				mode = MODE_NORMAL;
			} else if ((cmd[0] == 'n') && (cmd[1] == 'm')) {
				host_n  = v;
				host_dp = 4 - d;
				mode = MODE_HOST_N;
			} else if ((cmd[0] == 't') && (cmd[1] == 'm')) {
				mode = MODE_NORMAL;
			} else if ((cmd[0] == 'h') && (cmd[1] == 'h')) {
				if (v == 12) {
					ee.hh12 = 1;
				} else {
					ee.hh12 = 0;
				}
				ee.update = 1;
			} else if ((cmd[0] == 'a') && (cmd[1] == 'j')) {
				ee.tmr0val += v;
//				ee.tmr0val -= (((long)ee.tmr0val * v) / (24L * 60L * 60L));
				ee.update = 1;
				r = ~ee.tmr0val;
			} else if ((cmd[0] == 'd') && (cmd[1] == 'h')) {
				ee.dim_hi_hh = v;
				ee.update    = 1;
			} else if ((cmd[0] == 'd') && (cmd[1] == 'l')) {
				ee.dim_lo_hh = v;
				ee.update    = 1;
			} else if (cmd[0] == '?') {
				const rom char *msg[] = {
					"st1234:set time\r\n",
					"nm9999:num mode\r\n",
					"tm:time mode\r\n",
					"hh12/24:Display\r\n",
					"aj-99:Fine adjust\r\n",
//					"aj-99:Adj sec/day\r\n",
					"dh99:Dimmer high hh\r\n",
					"dl99:Dimmer low hh\r\n",
					NULL
				};
				unsigned char	n;

				for(n = 0; ; n++) {
					if (msg[n] == NULL) break;
					strcpypgm2ram(cmd, msg[n]);
					usb_puts(cmd);
				}
			} else {
				f = 0;
				r = 1;
			}

			// Send response
			if (f) {
				cmd[0] = 'o';
				cmd[1] = 'k';
			} else {
				cmd[0] = 'n';
				cmd[1] = 'g';
			}
			utoanm(r, &cmd[2] , 6, -1);
			cmd[8] = '\r';
			cmd[9] = '\n';
			cmd[10] = '\0';
			usb_puts(cmd);
		}

		// Save to EEPROM
		if (ee.update){
			unsigned char	*p;
			unsigned int	a;

			ee.update = 0;
			p = (unsigned char *)&ee;
			for(a = 0; a < sizeof(ee); a++){
				Write_b_eep(a, *p++);
				Busy_eep();
			}
		}

		// Wait event
		Sleep();
		Nop();
		ClrWdt();

	}
}
